"""Base class implementation for that has JavaScript dictionary
data structure.
"""


class DictionaryStructure:
    """
    Base class implementation for that has JavaScript dictionary
    data structure.

    Instance that inherite this base class will be treated as
    dictionary value in JavaScript (e.g., assert_equal dictionary
    type branch instruction).
    """
